(* some command line argument state *)
let patches = ref []
let add_patch patch = patches := patch :: !patches
let filesdir = ref ""
let replace = ref false
let camlsyntax = ref false
let version = ref false

let speclist = [
    ("-patch", Arg.String add_patch, "<patch> includes the given patch");
    ("-filesdir", Arg.Set_string filesdir,
                    "<dir> includes the given heirarchy of files");
    ("-replace", Arg.Set replace, "replaces an old package that is in the way");
    ("-camlsyntax", Arg.Set camlsyntax, "use the ocaml syntax (devel only)");
    ("-version", Arg.Set version, "display version information and exit");
]

(* some more command line argument state *)
let anonargs = ref []
let anonfun arg = anonargs := arg :: !anonargs

let usage_msg = "Usage: godiva [options] spec godi_localbase"

let version () =
    Log.info ("This is GODIVA version " ^ Version.v ^ ".");
    if !version then exit 0 else ()

let () =
    Arg.parse speclist anonfun usage_msg;
    version ();
    match (List.rev !anonargs) with
      [ spec_file; godi_localbase ] ->
        if !camlsyntax then
          begin
            Log.info "Using O'Caml syntax for spec file";
            (* some rather nasty hacks to tack a header and footer on *)
            let tmpfile = Filename.temp_file "godiva-mlspec" ".ml" in
            let tmpfile_chan = open_out tmpfile in
            output_string tmpfile_chan "open Il\nlet spec = {\n";
            let mlspec_chan = open_in spec_file in
            let il_file_contents =
                let rec input_file () =
                  try
                    begin
                     let line = input_line mlspec_chan in
                     line :: (input_file ())
                    end
                  with End_of_file -> [""]
                in
                String.concat "\n" (input_file ())
            in
            output_string tmpfile_chan il_file_contents;
            close_in mlspec_chan;
            output_string tmpfile_chan
                "}\nlet () = Dynload.spec := Some spec\n";
            close_out tmpfile_chan;
            (* all done, the thing to compile is now in tmpfile *)
            let command = "ocamlfind ocamlc -package godiva -c " ^ tmpfile in
            print_endline command;
            (if Sys.command command <> 0 then
              begin
                Log.error ("Unable to compile the spec file for "
                         ^ "dynamic loading.  Come again another day.");
                Unix.unlink tmpfile;
                exit 1
              end);
            (* basename really could have the directory in front here: *)
            let dotcmo = (Filename.chop_extension tmpfile) ^ ".cmo" in
            let dotcmi = (Filename.chop_extension tmpfile) ^ ".cmi" in
            (try
                Dynlink.init ();
                (* this sets Dynload.spec side effectually *)
                Dynlink.loadfile dotcmo;
                (* now cleanup *)
                Unix.unlink dotcmo;
                Unix.unlink dotcmi;
                Unix.unlink tmpfile
            with Dynlink.Error e -> Log.error (Dynlink.error_message e);
                                    exit 1);
            (match !Dynload.spec with
              None -> assert false
            | Some spec -> let godi_package = Translate.f spec in
                           Output.f ~godi_localbase ~godi_package
                                                    ~replace:!replace);
            ()
          end
        else (* normal syntax *)
          begin
            let lexbuf = Lexer.open_lex spec_file in
            Log.info "parsing spec file..";
            let el = Syntax.handle (fun () -> Parser.spec Lexer.token lexbuf) in
            Log.info "performing semantic analysis..";
            let filesdir = match !filesdir with "" -> None | fd -> Some fd in
            let il = Semanticanalyzer.f ~el ~patches:!patches ~filesdir in
            Log.info "translating to godi package..";
            let godi_package = Translate.f il in
            Log.info "outputting..";
            Output.f ~godi_localbase ~godi_package ~replace:!replace;
            ()
          end;

    | _ -> Arg.usage speclist usage_msg
